import * as HTTP from "http";
import * as ConcatStream from "concat-stream";
//import * as ZipStream from "zip-stream";
import * as Archiver from "archiver";
import * as FS from "fs";
import { Files } from "./Files";

export class Archive {
    
    public static async ArchiveFilesToMemory(location:string, files:Array<string>):Promise<Buffer> {
        let archive = new Archive();
        archive.fileNames = files;
        for(let i = 0; i < files.length; i++) {
            archive.fileLocations.push(location);
        }
        return await archive.GetBuffer();
    }

    public static async ArchiveFilesToResponse(location:string, files:Array<string>, response:HTTP.ServerResponse):Promise<void> {
        let archive = new Archive();
        archive.fileNames = files;
        for(let i = 0; i < files.length; i++) {
            archive.fileLocations.push(location);
        }
        return await archive.PipeToResponse(response);
    }

    // constructor
    public constructor() {
        //this.archiveStream = new ZipStream();
        this.archiveStream = new Archiver("zip");
    }

    // variables
    private fileNames:Array<string> = [];
    private fileLocations:Array<string> = [];
    private concatStream:any;
    private archiveStream:Packager;

    public AddFile(location:string):void { this.fileNames.push(location); }

    private async ArchiveFile(location:string, fileName:string):Promise<void> {
        //let contents = await Files.ReadFileToBuffer(location + "/" + fileName);
        return new Promise<void>((resolve, reject) => {
            /*
            this.archiveStream
            .entry(
                , 
                { name: fileName }, 
                function(err, entry) {
                    if(err != null) { return reject(err); }
                    resolve(entry);
                }
            );
            */
            this.archiveStream
            .append(
                Files.ReadStream(location + "/" + fileName), 
                { name: fileName },
                function(err, entry) {
                    if(err != null) { 
                        return reject(err); 
                    }
                    resolve(entry);
                }
            );
        });
    }

    public async GetBuffer():Promise<Buffer> {

        let self = this;
        return new Promise<Buffer>(async (resolve, reject) => {
            let concat = ConcatStream(function(result:Buffer) {
                resolve(result);
            });
            self.archiveStream.pipe(concat);
            for(let i = 0; i < self.fileNames.length; i++) {
                await self.ArchiveFile(self.fileLocations[i], self.fileNames[i]);
            }
            self.archiveStream.finish();
        });
    }

    public async PipeToResponse(response:HTTP.ServerResponse):Promise<void> {
        response.writeHead(200, {
            'Content-Type': 'application/zip'
        });
        let self = this;
        return new Promise<void>(async (resolve, reject) => {
            self.archiveStream.pipe(response);
            for(let i = 0; i < self.fileNames.length; i++) {
                self.archiveStream.append(Files.ReadStream(self.fileLocations[i] + "/" + self.fileNames[i]), { name: self.fileNames[i] }, null);
                //await self.ArchiveFile(self.fileLocations[i], self.fileNames[i]);
            }
            self.archiveStream.on("error", function(err) {
                throw err;
            })
            self.archiveStream.finalize();
            //response.end()
            resolve();
        });
    }
}

interface Packager {
    on(event:string, handler:Function):void;
    append(contentOrStream:string | FS.ReadStream, options:PackagerEntryOptions, callback:Function):void;
    entry(contents:Buffer, options:PackagerEntryOptions, callback:Function):void;
    finish():void;
    pipe(stream):void;
    finalize():void;
}

interface PackagerEntryOptions { name?:string; }