import * as Archiver from "archiver";
//import * as ZipStream from "zip-stream";
import * as FS from "fs";
import * as Streams from "stream";
import { Files } from "./Files";

import * as UnZip from "node-zip";
import * as Utilities from "./All";


export class Archives {

    public static async ZipDirectoryTo(src:string, dst:string):Promise<void> {

        return new Promise<void>((resolve, reject) => {
            let output = FS.createWriteStream(dst);
            let archive = Archiver("zip", {store:true});

            output.on("close", function(){
                resolve();
            });

            archive.on("error", function(err){
                reject(err);
            });

            archive.pipe(output);
            archive.directory(src, "");
            archive.finalize();
        });

    }

    public static async ZipDirectoryToBuffer(src:string):Promise<Buffer> {

        let tmp_file_location = "/tmp/" + Utilities.UUID() + ".zip";
        await Archives.ZipDirectoryTo(src, tmp_file_location);
        let buffer = await Utilities.Files.ReadFileToBuffer(tmp_file_location);
        await Utilities.Files.Delete(tmp_file_location);
        return buffer;
    }

    public static async UnZipBufferToDirectory(src:Buffer, dst:string):Promise<void> {
        
        let unzip = UnZip(src, {base64:false});
        let files = unzip.files;
        let keys = Object.keys(files);
        for(let i = 0; i < keys.length; i++) {
            let key = keys[i];
            let entry = files[key];
            if(entry.dir == false) {
                await Files.Write(dst + "/" + key, entry._data.getContent());
            }
        }
    }
}

class BufferStream extends Streams.Writable {

    private data:Array<Buffer> = [];

    constructor(options?) {
        super(options);
    }

    public _write(chunk:Buffer, enc:string, cb:Function):void {
        this.data.push(chunk);
        cb();
    }

    public GetBuffer():Buffer { return Buffer.concat(this.data); }
}