"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const FS = require("fs");
//import * as Path from "path";
class Files {
    static ReadUTF8(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            return new Promise((resolve, reject) => {
                FS.readFile(path, 'utf8', function (error, data) {
                    if (error != null) {
                        return reject(error);
                    }
                    return resolve(data);
                });
            });
        });
    }
    static Read(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            return new Promise((resolve, reject) => {
                FS.readFile(path, function (error, data) {
                    if (error != null) {
                        return reject(error);
                    }
                    return resolve(data);
                });
            });
        });
    }
    static ReadFileToBuffer(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            return new Promise((resolve, reject) => {
                FS.readFile(path, function (error, data) {
                    if (error != null) {
                        return reject(error);
                    }
                    return resolve(data);
                });
            });
        });
    }
    static Write(location, contents) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            yield Files.AssertFilePath(path);
            return new Promise((resolve, reject) => {
                FS.writeFile(path, contents, function (error) {
                    if (error != null) {
                        return reject(error);
                    }
                    return resolve();
                });
            });
        });
    }
    static Delete(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let exists = yield Files.Exists(location);
            if (exists == false) {
                return;
            }
            let lstat = yield Files.Stat(location);
            if (lstat.isDirectory()) {
                let files = yield Files.ReadDir(location);
                for (let i = 0; i < files.length; i++) {
                    yield Files.Delete(location + "/" + files[i]);
                }
                let path = Files.InterpretPath(location);
                return new Promise((resolve, reject) => {
                    FS.rmdir(path, function (error) {
                        if (error != null) {
                            return reject(error);
                        }
                        return resolve();
                    });
                });
            }
            else {
                let path = Files.InterpretPath(location);
                return new Promise((resolve, reject) => {
                    FS.unlink(path, function (error) {
                        if (error != null) {
                            return reject(error);
                        }
                        return resolve();
                    });
                });
            }
        });
    }
    static Exists(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            return new Promise((resolve, reject) => {
                FS.exists(path, function (exists) {
                    return resolve(exists);
                });
            });
        });
    }
    static MkDir(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            return new Promise((resolve, reject) => {
                FS.mkdir(path, function (err) {
                    if (err != null) {
                        return reject(err);
                    }
                    return resolve();
                });
            });
        });
    }
    static AssertFilePath(file_path) {
        return __awaiter(this, void 0, void 0, function* () {
            let tokens = file_path.split("/");
            // drop the file token
            tokens = tokens.slice(0, tokens.length - 1);
            let idx = 1;
            let finished = (idx >= tokens.length);
            while (finished == false) {
                let current_tokens = tokens.slice(0, idx + 1);
                let current_path = current_tokens.join("/");
                let exists = yield Files.Exists(current_path);
                if (!exists) {
                    yield Files.MkDir(current_path);
                }
                idx++;
                finished = (idx >= tokens.length);
            }
        });
    }
    static Stat(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            return new Promise((resolve, reject) => {
                FS.stat(path, function (error, stat) {
                    if (error != null) {
                        return reject(error);
                    }
                    return resolve(stat);
                });
            });
        });
    }
    static ReadDir(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let path = Files.InterpretPath(location);
            return new Promise((resolve, reject) => {
                FS.readdir(path, function (error, data) {
                    if (error != null) {
                        return reject(error);
                    }
                    return resolve(data);
                });
            });
        });
    }
    static ReadDirRecursively(location, relative_location, exclude) {
        return __awaiter(this, void 0, void 0, function* () {
            if (exclude != null && location.indexOf(exclude) >= 0) {
                return [];
            }
            if (relative_location == null) {
                relative_location = "";
            }
            else {
                relative_location = relative_location + "/";
            }
            let files = yield Files.ReadDir(location);
            let result = [];
            for (let i = 0; i < files.length; i++) {
                let stat = yield Files.Stat(location + "/" + files[i]);
                if (stat.isFile()) {
                    result.push({
                        path: relative_location + files[i],
                        stat: stat
                    });
                    continue;
                }
                if (stat.isDirectory()) {
                    let dir_files = yield Files.ReadDirRecursively(location + "/" + files[i], relative_location + files[i], exclude);
                    result = result.concat(dir_files);
                    continue;
                }
            }
            return result;
        });
    }
    static ReadStream(location) {
        let path = Files.InterpretPath(location);
        return FS.createReadStream(path);
    }
    static Copy(src, dst) {
        return __awaiter(this, void 0, void 0, function* () {
            let contents = yield Files.Read(src);
            yield Files.Write(dst, contents);
        });
    }
    static InterpretPath(path) {
        if (!path.startsWith("/")) {
            path = this.RelativePath() + path;
        }
        let tokens = path.split("/");
        let result = [];
        for (let i = 0; i < tokens.length; i++) {
            if (tokens[i] == "..") {
                result.pop();
            }
            else {
                result.push(tokens[i]);
            }
        }
        return result.join("/");
    }
    static RelativePath() {
        return __dirname.substring(0, __dirname.lastIndexOf("/") + 1);
    }
}
exports.Files = Files;
//# sourceMappingURL=Files.js.map