"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const SERVER_LOCATION = "localhost:8080";
const Busboy = require("busboy");
const HTTP = require("http");
const FormData = require("form-data");
const QueryString = require("querystring");
const Utilities = require("../../utilities/All");
class ServerRequest {
    static Get(path, query, localRequest, localResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            var sendPath = "/service" + path;
            if (query != null) {
                sendPath = sendPath + "?" + QueryString.stringify(query);
            }
            // send the headers of the local request along (we're doing this proxy style)
            return new Promise((resolve, reject) => {
                let content = null;
                var request = HTTP.request({
                    method: "GET",
                    headers: localRequest.headers,
                    hostname: SERVER_LOCATION.split(":")[0],
                    port: Number.parseInt(SERVER_LOCATION.split(":")[1]),
                    path: sendPath
                }, function (response) {
                    response.setEncoding('utf8');
                    response.on('data', (chunk) => {
                        if (content == null) {
                            content = "";
                        }
                        content = content + chunk;
                    });
                    response.on('end', () => {
                        resolve({
                            request: response,
                            content: content
                        });
                    });
                });
                request.on("error", function (err) {
                    reject(err);
                });
                request.end();
            });
        });
    }
    static GetWithHeaders(path, query, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            var sendPath = "/service" + path;
            if (query != null) {
                sendPath = sendPath + "?" + QueryString.stringify(query);
            }
            // send the headers of the local request along (we're doing this proxy style)
            return new Promise((resolve, reject) => {
                let content = null;
                var request = HTTP.request({
                    method: "GET",
                    headers: headers,
                    hostname: SERVER_LOCATION.split(":")[0],
                    port: Number.parseInt(SERVER_LOCATION.split(":")[1]),
                    path: sendPath
                }, function (response) {
                    response.setEncoding('utf8');
                    response.on('data', (chunk) => {
                        if (content == null) {
                            content = "";
                        }
                        content = content + chunk;
                    });
                    response.on('end', () => {
                        resolve({
                            request: response,
                            content: content
                        });
                    });
                });
                request.on("error", function (err) {
                    reject(err);
                });
                request.end();
            });
        });
    }
    static PostWithHeaders(path, query, headers, json, files) {
        return __awaiter(this, void 0, void 0, function* () {
            // form sender
            var formData = new FormData();
            // parse form data using the keys
            formData.append("data", JSON.stringify(json));
            let keys = Object.keys(files);
            for (let i = 0; i < keys.length; i++) {
                let key = keys[i];
                let value = files[key];
                formData.append(value.name, value.data, { filename: value.filename });
            }
            var customHeaders = formData.getHeaders();
            if (customHeaders == null) {
                customHeaders = {};
            }
            if (headers != null) {
                let keys = Object.keys(headers);
                for (let i = 0; i < keys.length; i++) {
                    let key = keys[i];
                    let value = headers[key];
                    if (key.trim().toLowerCase().startsWith("content-type")) {
                        continue;
                    }
                    customHeaders[key] = value;
                }
            }
            /*
    
            let options = {
                headers: headers
            };
            */
            var sendPath = "/service" + path;
            if (query != null) {
                sendPath = sendPath + "?" + QueryString.stringify(query);
            }
            return new Promise((resolve, reject) => {
                let content = null;
                /*
                let request = REQUEST.post("http://" + SERVER_LOCATION + sendPath, options, function(err, response, body){
                    if(err != null) { return reject(err); }
                    resolve(response);
                });
                request.form(form);
                */
                var request = HTTP.request({
                    method: 'post',
                    hostname: SERVER_LOCATION.split(":")[0],
                    port: Number.parseInt(SERVER_LOCATION.split(":")[1]),
                    path: sendPath,
                    headers: customHeaders
                }, function (response) {
                    response.setEncoding('utf8');
                    response.on('data', (chunk) => {
                        if (content == null) {
                            content = "";
                        }
                        content = content + chunk;
                    });
                    response.on('end', () => {
                        resolve({
                            request: response,
                            content: content
                        });
                    });
                });
                request.on("error", function (err) {
                    reject(err);
                });
                formData.pipe(request);
                //request.end();
            });
        });
    }
    static Proxy(from, to) {
        return __awaiter(this, void 0, void 0, function* () {
            // copy cookies
            Utilities.Web.CopyCookies(from.request, to);
            to.statusCode = from.request.statusCode;
            to.statusMessage = from.request.statusMessage;
            to.end(from.content);
        });
    }
    static ProxyGetRequest(url, localRequest, localResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            var queryParameters = yield Utilities.Web.GetRequestParameter(localRequest);
            var serverResponse = yield ServerRequest.Get(url, { data: JSON.stringify(queryParameters.fields) }, localRequest);
            yield ServerRequest.Proxy(serverResponse, localResponse);
        });
    }
    static ProxyPostRequest(url, localRequest, localResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            var queryParameters = yield Utilities.Web.GetRequestParameter(localRequest);
            var serverResponse = yield ServerRequest.Get(url, { data: JSON.stringify(queryParameters.fields) }, localRequest);
            yield ServerRequest.Proxy(serverResponse, localResponse);
        });
    }
    static ParseFormData(request) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                let fields = {};
                let files = {};
                var busboy = new Busboy({ headers: request.headers });
                busboy.on('file', function (fieldname, file, filename, encoding, mimetype) {
                    let fileBuffer = new Buffer("");
                    file.on('data', function (data) {
                        fileBuffer = Buffer.concat([fileBuffer, data]);
                    });
                    file.on('end', function () {
                        files[fieldname] = { path: null, data: fileBuffer, originalFilename: filename };
                    });
                });
                busboy.on('field', function (fieldname, val, fieldnameTruncated, valTruncated, encoding, mimetype) {
                    fields[fieldname] = val;
                });
                busboy.on('finish', function () {
                    resolve({
                        fields: fields,
                        files: files
                    });
                });
                request.pipe(busboy);
            });
        });
    }
    static ParseJsonData(request) {
        return __awaiter(this, void 0, void 0, function* () {
            let content = null;
            return new Promise((resolve, reject) => {
                request.on('data', function (chunk) {
                    if (content == null) {
                        content = "";
                    }
                    content = content + chunk;
                });
                request.on('end', function () {
                    resolve(JSON.parse(content));
                });
                request.on('error', function (err) {
                    reject(err);
                });
            });
        });
    }
}
exports.ServerRequest = ServerRequest;
//# sourceMappingURL=ServerRequest.js.map