"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HTTP = require("http");
const Utilities = require("../../utilities/All");
const Routes = require("./routes/All");
const DEFAULT_PORT = 8017;
class WebServer {
    Start() {
        return __awaiter(this, void 0, void 0, function* () {
            let self = this;
            // request handler
            this.http_server = HTTP.createServer(function (request, response) { self.HandleRequest(request, response); });
            return new Promise((resolve, reject) => {
                // listenting handler
                self.http_server.on('listening', function () { resolve(true); });
                // error handler
                self.http_server.on('error', function (err) {
                    if (err.code == 'EADDRINUSE') {
                        self.http_server.close();
                        resolve(false);
                    }
                    else {
                        reject(err);
                    }
                });
                try {
                    // start it up
                    self.http_server.listen(DEFAULT_PORT);
                }
                catch (err) {
                    resolve(false);
                }
            });
        });
    }
    HandleRequest(request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            // Set CORS headers
            response.setHeader('Access-Control-Allow-Origin', '*');
            response.setHeader('Access-Control-Request-Method', '*');
            response.setHeader('Access-Control-Allow-Methods', 'OPTIONS, GET');
            response.setHeader('Access-Control-Allow-Headers', '*');
            //response.setHeader('Access-Control-Allow-Headers', request.headers.origin);
            if (request.method === 'OPTIONS') {
                response.writeHead(200);
                response.end();
                return;
            }
            var path = Utilities.Web.GetPath(request);
            if (path.startsWith("/service")) {
                // route / handle as service
                path = path.substring("/service".length);
                yield Routes.Serve(path, request, response);
            }
            else if (path.startsWith("/file")) {
                path = path.substring("/file/".length);
                yield this.ServeFile(path, response);
            }
            else {
                // serve the index file
                yield this.ServeFile("index.test.html", response);
            }
        });
    }
    ServeFile(path, response) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (path.startsWith("__framework__/")) {
                    path = "../builders/web-application-builder/resources/" + path.substring("__framework__/".length);
                }
                else {
                    path = "web/client/" + path;
                }
                if (path.indexOf("") >= 0) {
                    console.log("path = " + path);
                }
                var buffer = yield Utilities.Files.Read(path);
                response.write(buffer);
                response.end();
            }
            catch (err) {
                response.statusCode = 404;
                response.end();
            }
        });
    }
}
exports.WebServer = WebServer;
//# sourceMappingURL=WebServer.js.map