import * as HTTP from "http";
import * as Utilities from "../../utilities/All";
import * as Routes from "./routes/All";

const DEFAULT_PORT:number = 8017;

export class WebServer {

    private http_server:HTTP.Server;

    public async Start():Promise<boolean> {

        let self = this;
        // request handler
        this.http_server = HTTP.createServer(function(request, response){ self.HandleRequest(request, response); } );

        return new Promise<boolean>((resolve, reject) => {
            
            // listenting handler
            self.http_server.on('listening', function(){ resolve(true); });

            // error handler
            self.http_server.on('error', function(err:any){
                if(err.code == 'EADDRINUSE') {
                    self.http_server.close();
                    resolve(false);
                } else {
                    reject(err);
                }
            });

            try{
                // start it up
                self.http_server.listen(DEFAULT_PORT);
            } catch(err) {
                resolve(false);
            }
        });
    }

    private async HandleRequest(request:HTTP.IncomingMessage, response:HTTP.ServerResponse):Promise<void> {

        // Set CORS headers
        response.setHeader('Access-Control-Allow-Origin', '*');
        response.setHeader('Access-Control-Request-Method', '*');
        response.setHeader('Access-Control-Allow-Methods', 'OPTIONS, GET');
        response.setHeader('Access-Control-Allow-Headers', '*');
        //response.setHeader('Access-Control-Allow-Headers', request.headers.origin);
        if ( request.method === 'OPTIONS' ) {
            response.writeHead(200);
            response.end();
            return;
        }

        var path = Utilities.Web.GetPath(request);
        if(path.startsWith("/service")) 
        {
            // route / handle as service
            path = path.substring("/service".length);
            await Routes.Serve(path, request, response);
        } 
        else if(path.startsWith("/file")) 
        {
            path = path.substring("/file/".length);
            await this.ServeFile(path, response);
        } 
        else 
        {
            // serve the index file
            await this.ServeFile("index.test.html", response);
        }
    }

    private async ServeFile(path:string, response:HTTP.ServerResponse):Promise<void> {

        try {

            if(path.startsWith("__framework__/")) {
                path = "../builders/web-application-builder/resources/" + path.substring("__framework__/".length);
            } else {
                path = "web/client/" + path;
            }

            if(path.indexOf("") >= 0) {
                console.log("path = " + path);
            }

            var buffer = await Utilities.Files.Read(path);
            response.write(buffer);
            response.end();

        } catch(err) {
            response.statusCode = 404;
            response.end();
        }
    }
}