"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utilities = require("../../../utilities/All");
const ServerRequest_1 = require("../ServerRequest");
//import * as MultiParty from "multiparty";
const Crypto = require("crypto");
//import * as Archiver from "archiver";
class Build {
    static BuildLocal(request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            //var queryParameters = await ServerRequest.ParseFormData(request);
            //var queryParameters = await Utilities.Web.GetRequestParameter(request);
            //var fields = queryParameters.fields;
            var fields = yield ServerRequest_1.ServerRequest.ParseJsonData(request);
            let source = null;
            let module = null;
            let hashSet = null;
            if (fields.module != null && fields.source != null) {
                module = fields.module;
                source = fields.source;
                hashSet = fields.hashSet;
            }
            let meta = yield Build.GetMeta(module, source, hashSet);
            if (meta.project_files.length > 0 || meta.missing_files.length > 0) {
                let files = meta.project_files.slice();
                yield Utilities.Files.Write(source.Location + "/.framework.sh.meta", JSON.stringify(meta));
                files.push(".framework.sh.meta");
                yield Utilities.Archive.ArchiveFilesToResponse(source.Location, files, response);
                yield Utilities.Files.Delete(source.Location + "/.framework.sh.meta");
                return;
            }
            response.writeHead(200, {
                'Content-Type': 'application/zip',
                'Content-Length': 0
            });
            return new Promise((resolve, reject) => {
                response.end(function () {
                    resolve();
                });
                response.on('error', function (err) {
                    reject(err);
                });
            });
        });
    }
    static BuildUpdate(request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            //var queryParameters = await ServerRequest.ParseJsonData(request);
            //var queryParameters = await Utilities.Web.GetRequestParameter(request);
            //var fields = queryParameters.fields;
            var fields = yield ServerRequest_1.ServerRequest.ParseJsonData(request);
            let source = fields.source;
            let module = fields.module;
            let build = fields.build;
            if (build.FrameworkDTS != null) {
                let frameworkDTS = null;
                try {
                    frameworkDTS = yield Utilities.Files.ReadUTF8(source.Location + "/framework.d.ts");
                }
                catch (err) { }
                if (frameworkDTS == null || frameworkDTS.trim() != build.FrameworkDTS.trim()) {
                    yield Utilities.Files.Write(source.Location + "/framework.d.ts", build.FrameworkDTS);
                }
            }
            let json = JSON.stringify(null);
            response.writeHead(200, {
                'Content-Type': 'application/json',
                'Content-Length': json.length
            });
            return new Promise((resolve, reject) => {
                response.write(json);
                response.end(function () {
                    resolve();
                });
                response.on('error', function (err) {
                    reject(err);
                });
            });
        });
    }
    static GetMeta(module, source, remoteHashSet) {
        return __awaiter(this, void 0, void 0, function* () {
            var location = source.Location;
            var fileArray = yield Utilities.Files.ReadDirRecursively(location, null, "node_modules");
            // get a copy of the local hash set
            // if a local one does not exist then read / hash the files
            let localHashSet = Build.HashSets[module.id];
            let modifiedTimestamps = Build.ModifiedTimestamps[module.id];
            let localHashSetNew = false;
            if (localHashSet == null) {
                localHashSetNew = true;
                localHashSet = {
                    Paths: [],
                    Hashes: {}
                };
                modifiedTimestamps = {
                    Timestamps: {}
                };
                let promises = new Array(fileArray.length);
                for (let i = 0; i < fileArray.length; i++) {
                    let path = fileArray[i].path;
                    // timestamp
                    modifiedTimestamps.Timestamps[path] = fileArray[i].stat.mtime;
                    let promise = function () {
                        return __awaiter(this, void 0, void 0, function* () {
                            let hash = yield Build.HashFile(location + "/" + path);
                            localHashSet.Paths.push(path);
                            localHashSet.Hashes[path] = hash;
                        });
                    };
                    promises[i] = promise();
                    /*
                    // hash
                    let contents = await Utilities.Files.ReadFileToBuffer(location + "/" + path);
                    let hash = RevHash(contents);
                    localHashSet.Paths.push(path);
                    localHashSet.Hashes[path] = hash;
    
                    // timestamp
                    modifiedTimestamps.Timestamps[path] = fileArray[i].stat.mtime;
                    */
                }
                yield Promise.all(promises);
                Build.HashSets[module.id] = localHashSet;
                Build.ModifiedTimestamps[module.id] = modifiedTimestamps;
            }
            // by looping over the local files
            // we can detect new / modified files
            let newModifiedFiles = [];
            let fileMap = {};
            let changeDetectedPromises = [];
            for (let i = 0; i < fileArray.length; i++) {
                let file = fileArray[i];
                let timestamp = modifiedTimestamps.Timestamps[file.path];
                let localHash = localHashSet.Hashes[file.path];
                let remoteHash = remoteHashSet == null ? null : remoteHashSet.Hashes[file.path];
                fileMap[file.path] = -1;
                // actively scrub .js and .js.map files
                let isTs = file.path.endsWith(".ts");
                let isJs = file.path.endsWith(".js");
                let isMap = file.path.endsWith(".js.map");
                if (isTs || isJs || isMap) {
                    let nameOnly = file.path;
                    if (isTs) {
                        nameOnly = nameOnly.substring(0, nameOnly.lastIndexOf(".ts"));
                    }
                    else if (isJs) {
                        nameOnly = nameOnly.substring(0, nameOnly.lastIndexOf(".js"));
                    }
                    else if (isMap) {
                        nameOnly = nameOnly.substring(0, nameOnly.lastIndexOf(".js.map"));
                    }
                    // is TS file (scrub JS / MAP and continue)
                    if (isTs) {
                        let jsIdx = fileMap[nameOnly + ".js"];
                        let mapIdx = fileMap[nameOnly + ".js.map"];
                        if (jsIdx != null && jsIdx >= 0) {
                            newModifiedFiles[jsIdx] = null;
                        }
                        if (mapIdx != null && mapIdx >= 0) {
                            newModifiedFiles[mapIdx] = null;
                        }
                        if (localHashSet.Hashes[nameOnly + ".js"] != null) {
                            delete localHashSet.Hashes[nameOnly + ".js"];
                            delete modifiedTimestamps.Timestamps[nameOnly + ".js"];
                        }
                        if (localHash[nameOnly + ".js.map"] != null) {
                            delete localHashSet.Hashes[nameOnly + ".js.map"];
                            delete modifiedTimestamps.Timestamps[nameOnly + ".js.map"];
                        }
                    }
                    // is JS file (scrub MAP and continue)
                    else if (isJs) {
                        let tsIdx = fileMap[nameOnly + ".ts"];
                        let tsFileFound = tsIdx != null;
                        if (tsFileFound) {
                            let mapIdx = fileMap[nameOnly + ".js.map"];
                            if (mapIdx != null && mapIdx >= 0) {
                                newModifiedFiles[mapIdx] = null;
                            }
                            if (localHash[nameOnly + ".js.map"] != null) {
                                delete localHashSet.Hashes[nameOnly + ".js.map"];
                                delete modifiedTimestamps.Timestamps[nameOnly + ".js.map"];
                            }
                            continue;
                        }
                    }
                    // is MAP file (scrub JS and continue)
                    else if (isMap) {
                        let tsIdx = fileMap[nameOnly + ".ts"];
                        let tsFileFound = tsIdx != null;
                        if (tsFileFound) {
                            let jsIdx = fileMap[nameOnly + ".js"];
                            if (jsIdx != null && jsIdx >= 0) {
                                newModifiedFiles[jsIdx] = null;
                            }
                            if (localHash[nameOnly + ".js"] != null) {
                                delete localHashSet.Hashes[nameOnly + ".js"];
                                delete modifiedTimestamps.Timestamps[nameOnly + ".js"];
                            }
                            continue;
                        }
                    }
                }
                // this means the file is new || modified
                if (remoteHash == null || localHash != remoteHash || timestamp == null || file.stat.mtime == null || timestamp.getTime() < file.stat.mtime.getTime()) {
                    // update mtime / hash
                    modifiedTimestamps.Timestamps[file.path] = file.stat.mtime;
                    let promise = function () {
                        return __awaiter(this, void 0, void 0, function* () {
                            if (localHashSetNew == false) {
                                //let contents = await Utilities.Files.ReadFileToBuffer(location + "/" + file.path);
                                //let hash = RevHash(contents);
                                let hash = yield Build.HashFile(location + "/" + file.path);
                                localHashSet.Hashes[file.path] = hash;
                            }
                            newModifiedFiles.push(file.path);
                            fileMap[file.path] = newModifiedFiles.length - 1;
                        });
                    };
                    changeDetectedPromises.push(promise());
                }
            }
            // await the change detection (in parallel)
            yield Promise.all(changeDetectedPromises);
            // compress the newModifiedFiles
            let tempResult = [];
            for (let i = 0; i < newModifiedFiles.length; i++) {
                let file = newModifiedFiles[i];
                if (file == null) {
                    continue;
                }
                tempResult.push(file);
            }
            newModifiedFiles = tempResult;
            // by looping over the remote files
            // we can detect missing files (these must be deleted remotely)
            let missingFiles = [];
            if (remoteHashSet != null) {
                for (let i = 0; i < remoteHashSet.Paths.length; i++) {
                    let path = remoteHashSet.Paths[i];
                    if (localHashSet.Hashes[path] == null) {
                        console.log("$$missing: " + path);
                        missingFiles.push(path);
                    }
                }
            }
            return {
                project_files: newModifiedFiles,
                missing_files: missingFiles,
                hashes: localHashSet.Hashes
            };
        });
    }
    static HashFile(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let stream = Utilities.Files.ReadStream(location);
            let hash = Crypto.createHash("md5");
            return new Promise((resolve, reject) => {
                stream
                    .on("data", function (chunk) { hash.update(chunk); })
                    .on("end", function () { resolve(hash.digest("hex").slice(0, 10)); })
                    .on("error", function (err) { reject(err); });
            });
        });
    }
}
Build.HashSets = {};
Build.ModifiedTimestamps = {};
exports.Build = Build;
//# sourceMappingURL=Build.js.map